<?php

namespace Hrm\Timesheet\Policies;

use Litepie\User\Contracts\UserPolicy;
use Hrm\Timesheet\Models\Working;

class WorkingPolicy
{

    /**
     * Determine if the given user can view the working.
     *
     * @param UserPolicy $user
     * @param Working $working
     *
     * @return bool
     */
    public function view(UserPolicy $user, Working $working)
    {
        if ($user->canDo('timesheet.working.view') && $user->isAdmin()) {
            return true;
        }

        return $working->user_id == user_id() && $working->user_type == user_type();
    }

    /**
     * Determine if the given user can create a working.
     *
     * @param UserPolicy $user
     * @param Working $working
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('timesheet.working.create');
    }

    /**
     * Determine if the given user can update the given working.
     *
     * @param UserPolicy $user
     * @param Working $working
     *
     * @return bool
     */
    public function update(UserPolicy $user, Working $working)
    {
        if ($user->canDo('timesheet.working.edit') && $user->isAdmin()) {
            return true;
        }

        return $working->user_id == user_id() && $working->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given working.
     *
     * @param UserPolicy $user
     * @param Working $working
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Working $working)
    {
        return $working->user_id == user_id() && $working->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given working.
     *
     * @param UserPolicy $user
     * @param Working $working
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Working $working)
    {
        if ($user->canDo('timesheet.working.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given working.
     *
     * @param UserPolicy $user
     * @param Working $working
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Working $working)
    {
        if ($user->canDo('timesheet.working.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
