<?php

namespace Hrm\Timesheet\Providers;

use Illuminate\Support\ServiceProvider;

class TimesheetServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'timesheet');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'timesheet');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerTimesheet();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerTimesheet()
    {
        $this->app->bind('timesheet', function($app) {
            return new Timesheet($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Timesheet', 'Lavalite\Timesheet\Facades\Timesheet');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('hrm.timesheet', function ($app) {
            return $this->app->make('Hrm\Timesheet\Timesheet');
        });

        // Bind Timesheet to repository
        $this->app->bind(
            'Hrm\Timesheet\Interfaces\TimesheetRepositoryInterface',
            \Hrm\Timesheet\Repositories\Eloquent\TimesheetRepository::class
        );        // Bind Working to repository
        $this->app->bind(
            'Hrm\Timesheet\Interfaces\WorkingRepositoryInterface',
            \Hrm\Timesheet\Repositories\Eloquent\WorkingRepository::class
        );

        $this->app->register(\Hrm\Timesheet\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Hrm\Timesheet\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and timesheet's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/timesheet.php', 'timesheet'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeTimesheet::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['hrm.timesheet'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('hrm/timesheet.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/timesheet')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/timesheet')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
